<?php

namespace FiloBlu\Flow\Block\Adminhtml\Channel\Edit\Tab;

use FiloBlu\Flow\Block\Adminhtml\Channel\Renderer\OrderStatusFactory;
use FiloBlu\Flow\Block\Adminhtml\Option\MediaGalleryOptionSource;
use FiloBlu\Flow\Block\Adminhtml\Option\ProductAttributeOptionSource;
use FiloBlu\Flow\Helper\Data;
use FiloBlu\Flow\Model\Channel;
use FiloBlu\Flow\Model\Inboundflow;
use Magento\Backend\Block\Template\Context;
use Magento\Backend\Block\Widget\Form\Element\Dependence;
use Magento\Backend\Block\Widget\Form\Generic;
use Magento\Backend\Block\Widget\Tab\TabInterface;
use Magento\Framework\Data\Form;
use Magento\Framework\Data\FormFactory;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Registry;

/**
 * Class Config
 * @package FiloBlu\Flow\Block\Adminhtml\Channel\Edit\Tab
 */
class Config extends Generic implements TabInterface
{

    const SKU_CONVERSION_LOOKUP_DISABLED = 'disabled';
    const SKU_CONVERSION_LOOKUP_BARCODE = 'barcode';
    const SKU_CONVERSION_LOOKUP_EAN = 'ean';
    const SKU_CONVERSION_LOOKUP_FIELD = 'lookup_sku';

    const INDEXER_STRATEGY_NATIVE = 'native';
    const INDEXER_STRATEGY_FULL = 'full';
    const INDEXER_STRATEGY_FORCE = 'force';

    /**
     * @var OrderStatusFactory
     */
    protected $orderStatusFactory;
    /**
     * @var Data
     */
    protected $helper;
    /**
     * @var ProductAttributeOptionSource
     */
    private $productAttributeOptionSource;
    /**
     * @var MediaGalleryOptionSource
     */
    private $mediaGalleryOptionSource;

    /**
     * @param Data $helper
     * @param Context $context
     * @param Registry $registry
     * @param FormFactory $formFactory
     * @param OrderStatusFactory $orderStatusFactory
     * @param ProductAttributeOptionSource $productAttributeOptionSource
     * @param MediaGalleryOptionSource $mediaGalleryOptionSource
     * @param array $data
     */
    public function __construct(
        Data                         $helper,
        Context                      $context,
        Registry                     $registry,
        FormFactory                  $formFactory,
        OrderStatusFactory           $orderStatusFactory,
        ProductAttributeOptionSource $productAttributeOptionSource,
        MediaGalleryOptionSource     $mediaGalleryOptionSource,
        array                        $data = []
    )
    {
        parent::__construct($context, $registry, $formFactory, $data);
        $this->productAttributeOptionSource = $productAttributeOptionSource;
        $this->helper = $helper;
        $this->orderStatusFactory = $orderStatusFactory;
        $this->mediaGalleryOptionSource = $mediaGalleryOptionSource;
    }

    /**
     * Prepare label for tab
     *
     * @return string
     */
    public function getTabLabel()
    {
        return __('Config');
    }

    /**
     * Prepare title for tab
     *
     * @return string
     */
    public function getTabTitle()
    {
        return __('config');
    }

    /**
     * {@inheritdoc}
     */
    public function canShowTab()
    {
        return true;
    }

    /**
     * {@inheritdoc}
     */
    public function isHidden()
    {
        return false;
    }

    /**
     * Prepare form
     *
     * @return $this
     * @throws LocalizedException
     */
    protected function _prepareForm()
    {
        /** @var Channel $model */
        $model = $this->_coreRegistry->registry('flow_channel');

        /** @var Form $form */
        $form = $this->_formFactory->create();

        $form->setHtmlIdPrefix('page_');

        $fieldset = $form->addFieldset('config_fieldset', ['legend' => __('Config')]);

        $flow = $fieldset->addField(
            'flow',
            'select',
            [
                'name'     => 'config[flow]',
                'label'    => __('Model / Flow'),
                'required' => true,
                'values'   => $this->helper->getModels(),
                'note'     => 'Per Flow di tipo Multistock: installare anche filoblu/mod2-ext-msi'
            ]
        );

        $copyToIte = $fieldset->addField(
            'copy_to_ite',
            'select',
            [
                'name'     => 'config[copy_to_ite]',
                'label'    => __('Copy to Ite'),
                'required' => false,
                'values'   => [
                    '0' => 'No',
                    '1' => 'Yes'
                ],
                'default'  => 0,
                'note'     => 'Per Flow di tipo <b>Anagrafica</b>: duplicare il file per generare il corrispondente <b>Ite</b>?'
            ]
        );

        $processReservedStock = $fieldset->addField(
            'process_reserved_stock',
            'select',
            [
                'name'     => 'config[process_reserved_stock]',
                'label'    => __('Process Reserved Stock'),
                'required' => false,
                'values'   => [
                    '0' => 'No',
                    '1' => 'Yes'
                ],
                'default'  => '1'
            ]
        );

        $processReservedStockStatus = $fieldset->addField(
            'process_reserved_stock_status',
            'multiselect',
            [
                'name'     => 'config[process_reserved_stock_status]',
                'label'    => __('Add compensations when order status is'),
                'required' => false,
                'values'   => $this->orderStatusFactory->create()->toOptionArray()
            ]
        );

        $stockSource = $fieldset->addField('stock_source', 'select', [
            'label'   => __('Stock Source'),
            'name'    => 'config[stock_source]',
            'values'  => $this->helper->getSources(),
            'default' => 'default',
        ]);

        $discardNoManageStock = $fieldset->addField('discard_no_managed', 'select', [
            'label'   => __('Discard product without stock'),
            'name'    => 'config[discard_no_managed]',
            'values'  => [
                '0' => 'No',
                '1' => 'Yes'
            ],
            'default' => '0',
            'class'   => 'stock-channel-element'
        ]);

        $indexerStrategy = $fieldset->addField('indexer_strategy', 'select', [
            'label'   => __('Chose the indexer strategy'),
            'name'    => 'config[indexer_strategy]',
            'values'  => [
                self::INDEXER_STRATEGY_FULL   => self::INDEXER_STRATEGY_FULL,
                self::INDEXER_STRATEGY_NATIVE => self::INDEXER_STRATEGY_NATIVE,
                self::INDEXER_STRATEGY_FORCE  => self::INDEXER_STRATEGY_FORCE
            ],
            'default' => self::INDEXER_STRATEGY_FULL,
            'class'   => 'stock-channel-element'
        ]);

	    $fixInventorySourceItem = $fieldset->addField('fix_inventory_source_item', 'select', [
		    'label'   => __('Fix Inventory SourceItem'),
		    'name'    => 'config[fix_inventory_source_item]',
		    'values'  => [
				'1' => 'Yes',
			    '0' => 'No'
		    ],
		    'default' => '0',
		    'class'   => 'stock-channel-element',
		    'note'     => 'Remvoes all products differente form simple inside inventory_source_item table'
	    ]);

        $skuConversionLookup = $fieldset->addField(
            'sku_conversion_lookup',
            'select', [
            'label'   => __('SKU conversion lookup'),
            'name'    => 'config[sku_conversion_lookup]',
            'values'  => [
                self::SKU_CONVERSION_LOOKUP_DISABLED => self::SKU_CONVERSION_LOOKUP_DISABLED,
                self::SKU_CONVERSION_LOOKUP_BARCODE  => self::SKU_CONVERSION_LOOKUP_BARCODE,
                self::SKU_CONVERSION_LOOKUP_EAN      => self::SKU_CONVERSION_LOOKUP_EAN,
            ],
            'default' => self::SKU_CONVERSION_LOOKUP_DISABLED,
            'note'    => 'Questo attributo viene usato assieme al valore, se presente, della colonna \'lookup_sku\' (flow_from_stock) per calcolare lo SKU',
        ]);

        $preorderDiscard = $fieldset->addField(
            'preorder_discard',
            'select',
            [
                'name'     => 'config[preorder_discard]',
                'label'    => __('Discard Preorder'),
                'required' => false,
                'values'   => [
                    '0' => 'No',
                    '1' => 'Yes'
                ],
                'default'  => '0',
            ]
        );

        $preorderAttributeCode = $fieldset->addField(
            'preorder_attribute_code',
            'select',
            [
                'name'     => 'config[preorder_attribute_code]',
                'label'    => __('Preorder attribute'),
                'required' => false,
                'values'   => $this->productAttributeOptionSource->toOptionArray()
            ]
        );

        $preorderAttributeValue = $fieldset->addField(
            'preorder_attribute_value',
            'text',
            [
                'name'     => 'config[preorder_attribute_value]',
                'label'    => __('Preorder Attribute Value'),
                'required' => false
            ]
        );

        $iteChannelName = $fieldset->addField(
            'ite_channel_name',
            'text',
            [
                'name'     => 'config[ite_channel_name]',
                'label'    => __('Ite channel name'),
                'required' => false
            ]
        );

        $fieldset->addField('bus', 'select', [
            'label'   => __('Bus'),
            'name'    => 'config[bus]',
            'values'  => [
                '1' => '1',
                '2' => '2',
                '3' => '3',
                '4' => '4'
            ],
            'default' => '1'
        ]);

        $fieldset->addField('priority', 'select', [
            'label'   => __('Priority'),
            'name'    => 'config[priority]',
            'values'  => Inboundflow::getPriorityList(),
            'default' => Inboundflow::PRIORITY_LOW
        ]);

        $fieldset->addField('type', 'select', [
            'label'   => __('Channel Type'),
            'name'    => 'config[type]',
            'values'  => [
                'file'     => 'File',
                'image'    => 'Image',
                'video'    => 'Video',
                'document' => 'Document',
                'swatch'   => 'Swatch'
            ],
            'default' => 'file'
        ]);

        $imageMediaGallery = $fieldset->addField(
            'image_media_gallery',
            'select',
            [
                'name'     => 'config[image_media_gallery]',
                'label'    => __('Image Media Gallery'),
                'required' => false,
                'values'   => $this->mediaGalleryOptionSource->toOptionArray(),
                'default'  => 'media_gallery',
            ]
        );

        $this->setChild(
            'form_after',
            $this->getLayout()->createBlock(Dependence::class)
                ->addFieldMap($flow->getHtmlId(), $flow->getName())
                ->addFieldMap($copyToIte->getHtmlId(), $copyToIte->getName())
                ->addFieldMap($iteChannelName->getHtmlId(), $iteChannelName->getName())
                ->addFieldMap($discardNoManageStock->getHtmlId(), $discardNoManageStock->getName())
                ->addFieldMap($stockSource->getHtmlId(), $stockSource->getName())
                ->addFieldMap($indexerStrategy->getHtmlId(), $indexerStrategy->getName())
	            ->addFieldMap($fixInventorySourceItem->getHtmlId(), $fixInventorySourceItem->getName())
                ->addFieldMap($skuConversionLookup->getHtmlId(), $skuConversionLookup->getName())
                ->addFieldMap($preorderDiscard->getHtmlId(), $preorderDiscard->getName())
                ->addFieldMap($preorderAttributeCode->getHtmlId(), $preorderAttributeCode->getName())
                ->addFieldMap($preorderAttributeValue->getHtmlId(), $preorderAttributeValue->getName())
                ->addFieldMap($processReservedStockStatus->getHtmlId(), $processReservedStockStatus->getName())
                ->addFieldMap($processReservedStock->getHtmlId(), $processReservedStock->getName())
                ->addFieldMap($imageMediaGallery->getHtmlId(), $imageMediaGallery->getName())
                ->addFieldDependence($iteChannelName->getName(), $copyToIte->getName(), '1')
                ->addFieldDependence($processReservedStockStatus->getName(), $processReservedStock->getName(), '1')
                ->addFieldDependence($preorderAttributeCode->getName(), $preorderDiscard->getName(), '1')
                ->addFieldDependence($preorderAttributeValue->getName(), $preorderDiscard->getName(), '1')
                ->addFieldDependence($copyToIte->getName(), $flow->getName(), 'From\\Eavs')
                ->addFieldDependence($iteChannelName->getName(), $flow->getName(), 'From\\Eavs')
                ->addFieldDependence($discardNoManageStock->getName(), $flow->getName(), 'From\\Stock')
                ->addFieldDependence($stockSource->getName(), $flow->getName(), 'From\\Stock')
                ->addFieldDependence($indexerStrategy->getName(), $flow->getName(), 'From\\Stock')
                ->addFieldDependence($skuConversionLookup->getName(), $flow->getName(), 'From\\Stock')
                ->addFieldDependence($preorderDiscard->getName(), $flow->getName(), 'From\\Stock')
                ->addFieldDependence($preorderAttributeCode->getName(), $flow->getName(), 'From\\Stock')
                ->addFieldDependence($preorderAttributeValue->getName(), $flow->getName(), 'From\\Stock')
                ->addFieldDependence($processReservedStock->getName(), $flow->getName(), 'From\\Stock')
                ->addFieldDependence($processReservedStockStatus->getName(), $flow->getName(), 'From\\Stock')
	            ->addFieldDependence($fixInventorySourceItem->getName(), $flow->getName(), 'From\\Stock')
	            ->addFieldDependence($imageMediaGallery->getName(), $flow->getName(), 'Imagesflow')
        );

        $form->setValues($model->getData('config'));
        $this->setForm($form);

        return parent::_prepareForm();
    }

    /**
     * Check permission for passed action
     *
     * @param string $resourceId
     * @return bool
     */
    protected function _isAllowedAction($resourceId)
    {
        return $this->_authorization->isAllowed($resourceId);
    }
}
